#include "cquad.h"

double cquad_dqag(double f(), double a, double b, double epsabs, double epsrel, int key, double *abserr, int *neval, int *ier, int *last, int *iwork, double *work)
{
/******************************************************************************
***  Date written:         03/31/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Last revision date:   11/02/2012  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Author:    Gianluca Russo    (gianlurusso@fastwebnet.it)               ***
***             Salvatore Cuomo   (salvatore.cuomo@unina.it)                ***
***                                                                         ***
***             University of Naples "Federico II"                          ***
***             Dipartimento di Matematica e Applicazioni "R. Caccioppoli"  ***
***									    ***
*** 									    ***
***									    ***
*** This code is provided "as is", without any warranty of any kind,        ***
*** either expressed or implied, including but not limited to, any implied  ***
*** warranty of merchantibility or fitness for any purpose. In no event     ***
*** will any party who distributed the code be liable for damages or for    ***
*** any claim(s) by any other party, including but not limited to, any      ***
*** lost profits, lost monies, lost data or data rendered inaccurate,	    ***
*** losses sustained by third parties, or any other special, incidental or  ***	
*** consequential damages arising out of the use or inability to use the    ***
*** program, even if the possibility of such damages has been advised	    ***	
*** against. The entire risk as to the quality, the performance, and the    ***	
*** fitness of the program for any particular purpose lies with the party   ***
*** using the code.							    ***
***									    ***
*** This code, and any derivative of this code, may not be used in a        ***
*** commercial package without the prior explicit written permission of     ***
*** the authors. Verbatim copies of this code may be made and distributed   ***
*** in any medium, provided that this copyright notice is not removed or    ***
*** altered in any way. No fees may be charged for distribution of the      ***
*** codes, other than a fee to cover the cost of the media and a            ***
*** reasonable handling fee.						    ***
***-------------------------------------------------------------------------***
***  Purpose:                                                               ***
***                                                                         ***
***    The function provides a C interface to the fortran routine dqag      ***
***    of quadpack package.                                                 ***
***                                                                         ***
***    The function calculates an approximation result to a given definite  ***
***    integral i = integral of f over (a,b), hopefully satisfying          ***
***    following claim for accuracy:                                        ***
***                                                                         ***
***              abs(i - result) <= max(epsabs, epsrel * abs(i))            ***
***-------------------------------------------------------------------------***
***  Description:                                                           ***
***                                                                         ***
***    A simple globally adaptive integrator, using the strategy of Aind.   ***
***    It is possible to choose between 6 pairs of Gauss-Kronrod quadrature ***
***    formulae for the rule evaluation component. The pairs of high degree ***
***    of precision are suitable for handling integration difficulties due  ***
***    to a strongly oscillating integrand.                                 ***
***                                                                         ***
***    Double precision version.                                            ***
***-------------------------------------------------------------------------***
***  Input:                                                                 ***
***                                                                         ***
***    f        function subprogram defining the integrand function f(x)    ***
***                                                                         ***
***    a        lower limit of integration                                  ***
***                                                                         ***
***    b        upper limit of integration                                  ***
***                                                                         ***
***    epsabs   absolute accoracy requested                                 ***
***                                                                         ***
***    epsrel   relative accoracy requested                                 ***
***                                                                         ***
***    key      key for choice of local integration rule. Precisely a       ***
***             gauss-kronrod pair is used with:                            ***
***                                                                         ***
***                               7 - 15 points if key < 2                  ***
***                              10 - 21 points if key = 2                  ***
***                              15 - 31 points if key = 3                  ***
***                              20 - 41 points if key = 4                  ***
***                              25 - 51 points if key = 5                  ***
***                              30 - 61 points if key > 5                  ***
***-------------------------------------------------------------------------***
***  Output:                                                                ***
***                                                                         ***
***    res      approximation to the integral                               ***
***                                                                         ***
***    abserr   estimate of the modulus of the absolute error               ***
***                                                                         ***
***    neval    number of integrand evaluations                             ***
***                                                                         ***
***    ier      normal termination flag:                                    ***
***                                                                         ***
***               ier = 0 --> normal and reliable termination of the        ***
***                           function. It is assumed that the requested    ***
***                           accuracy has been achieved.                   ***
***                                                                         ***
***               ier = 1 --> maximum number of subdivisions allowed has    ***
***                           been achieved.                                ***
***                                                                         ***
***               ier = 2 --> the occurrence of roundoff error is detected  ***
***                           which prevents the requested tolerance from   ***
***                           being achieved.                               ***
***                                                                         ***
***               ier = 3 --> extremely bad integrand behaviour occurs at   ***
***                           some points of the integration interval.      ***
***                                                                         ***
***               ier = 6 --> the input is invalid because limit < 1, or    ***
***                           lenw < limit * 4, or epsabs <= 0 and          ***
***                           epsrel < max (50 * rel.mach.acc., 0.5d-28).   ***
***                           In all these events res, abserr, neval and    ***
***                           last are set to zero.                         ***
***                                                                         ***
***    last     number of subintervals actually produced in the subdivision ***
***             process                                                     ***
***-------------------------------------------------------------------------***
***  Workspace:                                                             ***
***                                                                         ***
***    limit    dimensioning value for iwork.                               ***
***             limit determines the maximum number of subintervals in      ***
***             the partition of the given integration interval (a,b)       ***
***             (limit > 0)                                                 ***
***                                                                         ***
***    lenw     dimensioning value for work.                                ***
***             (lenw must be at least limit * 4)                           ***
***                                                                         ***
***    iwork    vector of dimension at least limit, the first k elements    ***
***             of which contain pointers to the error estimates over the   ***
***             subintervals, such that work(limit * 3 + iwork(1)), ...,    ***
***             work(limit * 3 + iwork(k)) form a decreasing sequence, with ***
***             k = last if last <= (limit/2 + 2), and k = limit + 1 - last ***
***             otherwise                                                   *** 
***                                                                         ***
***    work     vector of dimension at least lenw.                          ***
***             On return:                                                  ***
***                work(1), ..., work(last) contain the left end points of  ***
***                  the subintervals in the partition of (a,b)             ***
***                work(limit + 1), ..., work(limit + last) contain the     ***
***                  right end points                                       ***
***                work(limit * 2 + 1), ..., work(limit * 2 + last) contain ***
***                  the integral approximations over the subintervals      ***
***                work(limit * 3 + 1), ..., work(limit * 3 + last) contain ***
***                  the error estimates                                    ***
***-------------------------------------------------------------------------***
***  References:      QUADPACK - Piessens, de Doncker, Uberhuber, Kahaner   ***
***                                                                         ***
***                   http://netlib.org                                     ***
******************************************************************************/
    
    int lenw = LENW, limit = LIMIT;
    double res;

    dqag_(f, &a, &b, &epsabs, &epsrel, &key, &res, abserr, neval, ier, &limit, &lenw, last, iwork, work);

    return res;
}
